# -*- python -*-
# $RCSfile: destinationparamwidget.py,v $
# $Revision: 1.16 $
# $Author: langer $
# $Date: 2006-10-06 20:41:35 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


# Widget for the Destination parameter.  Needs a separate file because
# it uses the fileselector, which in turn depends on stuff in
# parameterwidgets.py, which otherwise would be the obvious place for
# this.

from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import guitop
from oof2.common.IO import automatic
from oof2.common.IO import destinationparam
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import fileselector
from oof2.common.IO.GUI import gtklogger
from oof2.common.IO.GUI import parameterwidgets
import gtk

MSGWINDOW = "Message Window"

class DestinationParamWidget(parameterwidgets.ParameterWidget):
    # All instances share the same namelist.
    namelist = [MSGWINDOW]
    def __init__(self, param=None, scope=None, name=None):
        parameterwidgets.ParameterWidget.__init__(self, gtk.Frame(),
                                                  scope=scope, name=name)
        vbox = gtk.VBox(spacing=2)
        self.gtk.add(vbox)

        self.chooser = chooser.ChooserWidget(DestinationParamWidget.namelist,
                                             callback=self.chooserCB,
                                             name='Chooser')
        vbox.pack_start(self.chooser.gtk, expand=0, fill=0)

        bbox = gtk.HBox(spacing=2, homogeneous=True)
        vbox.pack_start(bbox, expand=0, fill=0)

        newbutton = gtk.Button("New...")
        gtklogger.setWidgetName(newbutton, "New")
        bbox.pack_start(newbutton, expand=1, fill=1)
        gtklogger.connect(newbutton, 'clicked', self.newCB)

        self.clearbutton = gtk.Button("Clear")
        gtklogger.setWidgetName(self.clearbutton, "Clear")
        bbox.pack_start(self.clearbutton, expand=1, fill=1)
        gtklogger.connect(self.clearbutton, 'clicked', self.clearCB)

        switchboard.requestCallbackMain('destinations changed',
                                        self.rebuild)
        self.rebuild()
        self.widgetChanged(1, interactive=0)

    def rebuild(self):
        self.chooser.update(DestinationParamWidget.namelist)

    def newCB(self, button):
        debug.mainthreadTest()
        newname = fileselector.fileSelector(ident="destwidget", mode="w",
                                            title="Data destination")
        if newname and newname not in DestinationParamWidget.namelist:
            DestinationParamWidget.namelist.append(newname)
            self.rebuild()
            self.just_set_value(newname)
            self.widgetChanged(1, interactive=1)
    def clearCB(self, button):
        DestinationParamWidget.namelist = [MSGWINDOW]
        self.rebuild()
        self.widgetChanged(1, interactive=1)
    def chooserCB(self, *args):
        self.widgetChanged(1, interactive=1)
    def just_set_value(self, value):
        if value == automatic.automatic:
            self.chooser.set_state(MSGWINDOW)
        else:
            self.chooser.set_state(value)
    def set_value(self, value):
        self.just_set_value(value)
        self.widgetChanged(1, interactive=0)        
    def get_value(self):
        val = self.chooser.get_value()
        if val == MSGWINDOW:
            return automatic.automatic
        return val
        

def _make_DestParamWidget(self, scope=None):
    return DestinationParamWidget(self, scope=scope, name=self.name)

destinationparam.DestinationParameter.makeWidget = _make_DestParamWidget
