# -*- python -*-
# $RCSfile: microstructuremenu.py,v $
# $Revision: 1.69 $
# $Author: langer $
# $Date: 2007-04-19 15:35:35 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from oof2.SWIG.common import activearea
from oof2.SWIG.common import switchboard
from oof2.SWIG.common import ooferror
from oof2.SWIG.image import oofimage
from oof2.common import enum
from oof2.common import debug
from oof2.common import pixelselection
from oof2.common import parallel_enable
from oof2.common import primitives
from oof2.common import quit
from oof2.common.IO import automatic
from oof2.common.IO import datafile
from oof2.common.IO import mainmenu
from oof2.common.IO import microstructureIO
from oof2.common.IO import oofmenu
from oof2.common.IO import parameter
from oof2.common.IO import whoville
import oof2.common.microstructure
from types import *

if parallel_enable.enabled():
    from oof2.common.IO import microstructureIPC

micromenu = mainmenu.OOF.addItem(oofmenu.OOFMenuItem(
    'Microstructure',
    help="Create and manipulate &micro; objects.",
    cli_only=1))
#######################

def newMicrostructure(menuitem, name,
                      width, height,
                      width_in_pixels, height_in_pixels):
    if width<=0 or height<=0 or width_in_pixels<=0 or height_in_pixels<=0:
        raise ooferror.ErrUserError("Negative size values are not allowed.")

    if parallel_enable.enabled():
        # For the rear-end guys
        microstructureIPC.msmenu.New_Parallel(name=name,
                                             width=width, height=height,
                                             width_in_pixels=width_in_pixels,
                                             height_in_pixels=height_in_pixels)

    # Serial mode & #0 in parallel mode
    ms = oof2.common.microstructure.Microstructure(
        name, primitives.iPoint(width_in_pixels,
                                height_in_pixels),
        primitives.Point(width, height)
        )


def msNameResolver(param, name):
    if param.automatic():
        nm = 'microstructure'
    else:
        nm = name
    return oof2.common.microstructure.microStructures.uniqueName(nm)
    
micromenu.addItem(
    oofmenu.OOFMenuItem(
    'New',
    callback=newMicrostructure,
    params=[
    whoville.AutoWhoNameParameter('name', resolver=msNameResolver,
                                  value=automatic.automatic,
                                  tip="Name of the new Microstructure."),
    parameter.FloatParameter('width', 1., tip='Width in physical units.'),
    parameter.FloatParameter('height', 1.,tip='Height in physical units.'),
    parameter.IntParameter('width_in_pixels', 10, tip='Width in pixels.'),
    parameter.IntParameter('height_in_pixels', 10, tip='Height in pixels.')
    ],
    help="Create a new Microstructure.",
    discussion="""
    <para>This command creates a new empty &micro; with no associated
    &images;.  To create a &micro; with an &image;, see <xref
    linkend='MenuItem:OOF.Microstructure.Create_From_Image'/> or <xref
    linkend='MenuItem:OOF.Microstructure.Create_From_ImageFile'/>.  To
    add an &image; to an existing &micro;, see <xref
    linkend='MenuItem:OOF.File.Load.Image'/>.</para>"""
    ))

############################

def renameMicrostructure(menuitem, microstructure, name):
    if parallel_enable.enabled():
        microstructureIPC.msmenu.Rename(microstructure=microstructure,name=name)
        return

    ms = oof2.common.microstructure.microStructures[microstructure]
    ms.reserve()
    aa = ms.getObject().activearea
    aa.begin_writing()
    try:
        aa.rename(name)
    finally:
        aa.end_writing()
    ps = ms.getObject().pixelselection
    ps.begin_writing()
    try:
        ps.rename(name)
    finally:
        ps.end_writing()
    ms.begin_writing()
    try:
        ms.rename(name, exclude=ms.getObject().name())
    finally:
        ms.end_writing()
        ms.cancel_reservation()

micromenu.addItem(
    oofmenu.OOFMenuItem('Rename',
                        callback=renameMicrostructure,
                        params=[
    parameter.StringParameter('microstructure', '',
                              tip='Old name for the microstructure.'),
    whoville.WhoNameParameter('name', value='', 
                              tip='New name for the microstructure.')
    ],
                        help="Rename a Microstructure.",
                        discussion="""<para>
                        Change the name of an existing &micro;.
                        </para>"""
                        ))

############################

def copyMicrostructure(menuitem, microstructure, name):
    if parallel_enable.enabled():
        microstructureIPC.msmenu.Copy(microstructure=microstructure,name=name)
        return

    ms = oof2.common.microstructure.microStructures[microstructure]
    ms.begin_reading()
    grouplist = []
    try:
        sourceMS = ms.getObject()
        newMS = sourceMS.nominalCopy(name)
        newMScontext = oof2.common.microstructure.microStructures[name]
        # First, copy images and load them to copied MS.
        for imagectxt in sourceMS.getImageContexts():
            sourceimage = imagectxt.getObject()
            immidge = sourceimage.clone(sourceimage.name())
            imagecontext = oofimage.imageContexts.add(
                [newMS.name(),immidge.name()], immidge,
                parent=newMScontext)
        # Copy pixel groups
        for grpname in sourceMS.groupNames():
            sourcegrp = sourceMS.findGroup(grpname)
            # Ignore "newness", all groups will be new.
            (newgrp, newness) = newMS.getGroup(grpname)
            newgrp.add(sourcegrp.members())
            newgrp.set_meshable(sourcegrp.is_meshable())
            grouplist.append(newgrp)
    finally:
        ms.end_reading()
        
    for g in grouplist:
        switchboard.notify("new pixel group", g)

micromenu.addItem(oofmenu.OOFMenuItem(
    'Copy',
    callback=copyMicrostructure,
    params=[parameter.StringParameter('microstructure', '',
                                      tip='The source microstructure.'),    
            parameter.AutomaticNameParameter('name', 
                                             resolver=msNameResolver,
                                             value=automatic.automatic,
                                         tip="Name of the new Microstructure.")
            ],
    help="Copy a Microstructure.",
    discussion=
    """<para>A copied &micro; has the same physical size and pixel
    size as the original &micro;, and includes
    <emphasis>copies</emphasis> of all &images; and <link
    linkend='Section:Concepts:Microstructure:Pixel_Group'>pixel
    groups</link> contained in the original.  It does
    <emphasis>not</emphasis> include any &skels; or &meshes; from the
    original.</para>"""
    ))

#############################

def deleteMicrostructure(menuitem, microstructure):
    if parallel_enable.enabled():
        microstructureIPC.msmenu.Delete(microstructure=microstructure)
    else:
        mscontext = oof2.common.microstructure.microStructures[microstructure]
        mscontext.lockAndDelete()

micromenu.addItem(oofmenu.OOFMenuItem(
    'Delete',
    callback=deleteMicrostructure,
    params=[
    parameter.StringParameter('microstructure', '',
                              tip='Name of the microstructure to be deleted.')],
    help="Destroy a Microstructure.",
    discussion=
    """<para>Delete a &micro; and all of its associated &images;,
    &skels;, and &meshes;.  Be really sure that you want to do this
    before you do it. </para>"""
    ))


#########################

def saveMicrostructure(menuitem, filename, mode, format, microstructure):
    quit.disallow()
    try:
        ms = oof2.common.microstructure.microStructures[microstructure]
        dfile = datafile.writeDataFile(filename, mode, format)
        microstructureIO.writeMicrostructure(dfile, ms)
        dfile.close()
    finally:
        quit.allow()

mainmenu.OOF.File.Save.addItem(oofmenu.OOFMenuItem(
    'Microstructure',
    callback=saveMicrostructure,
    ordering=30,
    params=[
    parameter.StringParameter('filename', tip="Name of the file."),
    parameter.StringParameter('mode', tip="'w' to (over)write and 'a' to append."),
    enum.EnumParameter('format', datafile.DataFileFormat, datafile.ASCII,
                       tip="File format."),
    whoville.WhoParameter('microstructure',
                          oof2.common.microstructure.microStructures,
                          tip=parameter.emptyTipString)],
    help="Save a Microstructure in a file.",
    discussion="""
    <para>Store a &micro; in a file in one of several <link
    linkend='Section:Concepts:FileFormats'><varname>formats</varname></link>.
    The file can be reloaded by <xref
    linkend='MenuItem:OOF.File.Load.Script'/> or <xref
    linkend='MenuItem:OOF.File.Load.Data'/>, depending on the file
    format.</para>
    """
   ))

def _fixmenu(*args):
    import sys
    if oof2.common.microstructure.microStructures.nActual() == 0:
        mainmenu.OOF.File.Save.Microstructure.disable()
    else:
        mainmenu.OOF.File.Save.Microstructure.enable()
_fixmenu()
    
switchboard.requestCallback(('new who', 'Microstructure'), _fixmenu)
switchboard.requestCallback(('remove who', 'Microstructure'), _fixmenu)
