// -*- C++ -*-
// $RCSfile: freedom.h,v $
// $Revision: 1.8 $
// $Author: langer $
// $Date: 2006-12-19 20:29:44 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */


class DegreeOfFreedom;

#ifndef FREEDOM_H
#define FREEDOM_H

#include <iostream>
#include <string>

typedef unsigned char DOFFLAG;
const DOFFLAG ACTIVE = 1;	// fixed value
const DOFFLAG DISCONNECTED = 2; // node is irrelevant

class DegreeOfFreedom {
private:
  DegreeOfFreedom(const DegreeOfFreedom&);
public:				// should be private, but swig complains
  DegreeOfFreedom(double, int);
  ~DegreeOfFreedom() {}
  friend class FEMesh;		// only a FEMesh can create or destroy a DoF
protected:
  double value_;
  int index_;
  DOFFLAG flags;
  int nfix;			// number of times this dof has been fixed
  int refcount_;
public:
  int dofindex() const { return index_;}

  double value() const { return value_; }
  double &value() { return value_; }

  void fix() { nfix++; }
  void unfix() { if(nfix > 0) nfix--; }
  void free() { nfix=0; }
  bool isfree() const { return nfix == 0; }
  void activate() { flags |= ACTIVE; }
  void deactivate() { flags &= ~ACTIVE; }
  bool active() const { return flags & ACTIVE; }
  bool disconnected() const { return flags & DISCONNECTED; }
};


std::ostream &operator<<(std::ostream &os, const DegreeOfFreedom &dof);

#endif
