# -*- python -*-
# $RCSfile: orientmapdisplay.py,v $
# $Revision: 1.4 $
# $Author: langer $
# $Date: 2007-05-30 20:25:17 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import coord
from oof2.SWIG.engine import angle2color
from oof2.SWIG.engine import orientationimage
from oof2.SWIG.orientationmap import orientmapdata
from oof2.common import color
from oof2.common import debug
from oof2.common import primitives
from oof2.common import registeredclass
from oof2.common.IO import display
from oof2.common.IO import parameter
from oof2.common.IO import xmlmenudump
from oof2.engine.IO import microstructuredisplay

class OrientationMapDisplay(display.DisplayMethod):
    def __init__(self, colorscheme):
        self.colorscheme = colorscheme
        display.DisplayMethod.__init__(self)
    def draw(self, gfxwindow, device):
        msobj = self.who().getObject(gfxwindow)
        data = orientmapdata.getOrientationMap(msobj)
        if data is not None:
            orientimage = orientmapdata.OrientMapImage(data, self.colorscheme)
            device.draw_image(orientimage, coord.Coord(0,0), msobj.size())
    def getTimeStamp(self, gfxwindow):
        msobj = self.who().getObject(gfxwindow)
        return max(display.DisplayMethod.getTimeStamp(self, gfxwindow),
                   orientmapdata.getOrientationMapTimestamp(msobj))

registeredclass.Registration(
    'Orientation Map',
    display.DisplayMethod,
    OrientationMapDisplay,
    ordering=2,
    params=[parameter.RegisteredParameter('colorscheme',
                                          angle2color.Angle2Color,
                                tip='Method for converting angles to colors.')
            ],
    layerordering=display.Planar(0.5),
    whoclasses = ('Microstructure',),
    tip="Display a Microstructure's Orientation Map, whether or not it's used by Material Properties.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/orientationmap/reg/orientationmapdisplay.xml'))
